/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext;

import java.util.Collection;
import java.util.Map;

import com.google.inject.AbstractModule;
import com.google.inject.Module;
import com.google.inject.TypeLiteral;

/**
 * Guice {@link Module} that holds the {@link EncodingProvider}s.
 * 
 * @author diewald
 */
public class EncodingProviderModule<T> extends AbstractModule {

	/** References to the generated prototype {@link EncodingProvider}s. */
	private Map<TypeLiteral<? extends T>, EncodingProvider<? extends T>> encodingProvidersMap;

	/** Constructor. */
	public EncodingProviderModule(
			Map<TypeLiteral<? extends T>, EncodingProvider<? extends T>> encodingProvidersMap) {
		this.encodingProvidersMap = encodingProvidersMap;
	}

	/** Returns the set of {@link EncodingProvider}s of this {@link Module}. */
	public Collection<EncodingProvider<? extends T>> getEncodingProviders() {
		return encodingProvidersMap.values();
	}

	/** Returns the protp{@link EncodingProvider}s of the given {@code encoding}. */
	public EncodingProvider<? extends T>
			getEncodingProviderOf(TypeLiteral<? extends T> encodingKey) {
		return encodingProvidersMap.get(encodingKey);
	}

	/** {@inheritDoc} */
	@Override
	protected void configure() {
		for(EncodingProvider<? extends T> provider : encodingProvidersMap.values()) {
			install(provider);
		}
	}
}
