/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext;

import static java.lang.System.currentTimeMillis;

import java.time.Duration;

import org.opt4j.core.optimizer.OptimizerIterationListener;

/**
 * Listener that prints the time spent on one interation.
 * 
 * @author diewald
 */
public class IterationTimeListener implements OptimizerIterationListener {

	/** Remembers the timestamp of the previous call. */
	private long prevIterationTimeStamp = currentTimeMillis();

	/** {@inheritDoc} */
	@Override
	public void iterationComplete(int iteration) {
		long currentTime = currentTimeMillis();
		long duration = currentTime - prevIterationTimeStamp;

		Duration readableDuration = Duration.ofMillis(duration);
		System.out.println("Duration of iteration " + iteration + ":");
		System.out.println("Days: " + readableDuration.toDaysPart());
		System.out.println("Hours: " + readableDuration.toHoursPart());
		System.out.println("Minutes: " + readableDuration.toMinutesPart());
		System.out.println("Seconds: " + readableDuration.toSecondsPart());

		prevIterationTimeStamp = currentTimeMillis();
	}
}
