/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.base.ui.annotation.view.AnnotationViewPartBase.isUpdateEnabled;
import static org.fortiss.tooling.base.ui.annotation.view.AnnotationViewPartBase.setUpdateEnabled;
import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.Optional;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.platform.hierarchic.model.tile.Tile;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.GatewayUnit;
import org.fortiss.af3.platform.model.MemoryUnit;
import org.fortiss.af3.platform.model.TransmissionUnit;
import org.fortiss.af3.task.model.Signal;
import org.fortiss.af3.task.model.Task;
import org.fortiss.af3.task.model.TaskInputPort;
import org.fortiss.af3.task.model.TaskOutputPort;
import org.fortiss.af3.task.model.allocation.ComponentToTaskAllocationEntry;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;

/**
 * {@link SystemModelAdapter} which abstracts AutoFOCUS3 models for the DSE.
 * 
 * @author diewald
 */
public class AF3SystemModelAdapter extends
		SystemModelAdapter<Task, ExecutionUnit, TaskOutputPort, TaskInputPort, GatewayUnit, TransmissionUnit, MemoryUnit> {

	/** References the method which returns the multicast targets (if available). */
	Method multicastGetTargets;

	/** Cache the Annotation View's update status. */
	private boolean annotationUpdateEnabled;

	/** Constructor. */
	public AF3SystemModelAdapter(SuperSet<Task> tasks,
			SuperSet<ComponentToTaskAllocationEntry> componentTaskAllocs,
			SuperSet<ExecutionUnit> execUnits, SuperSet<Signal> signals) throws Exception {
		// FIXME: Porting Code.
		Optional<ComponentToTaskAllocationEntry> anyEntry =
				getFirst(componentTaskAllocs.getEntries());
		if(!anyEntry.isPresent()) {
			throw new Exception("The set of component to task allocations is empty." +
					" It is needed by the DSE. Exiting.");
		}
		Optional<IModelElement> anyComponent = getFirst(anyEntry.get().getSourceElements());
		if(!anyComponent.isPresent()) {
			throw new Exception("At least one task is not associated with a Component." +
					" It is needed by the DSE. Exiting.");
		}
		// End: Porting code.

		this.taskObjects = tasks.getEntries();
		this.faultContainmentRegion = Tile.class;

		// Disable any annotation view update.
		annotationUpdateEnabled = isUpdateEnabled();
		setUpdateEnabled(false);

		// platformConnections = createPlatfromResourceConnections(execUnits);
		// TODO(#3264): Reintegrate the support for Component Design Libraries.
		// Multimap<Component, ITaskAdapter<Component>> componentToReplacementAdapter =
		// createComponentReplacementAdapters(systemParameters.getReplacementComponents());
		Multimap<Task, ITaskAdapter> componentToReplacementAdapter = HashMultimap.create();

		// TODO(#3264): The three lines below belong to this issue.
		// componentReplacementAdapters = componentToReplacementAdapter.values();
		nonSafetyFunctionTasks = Collections.emptySet();// new HashSet<>(tasks);
		taskInstances = HashMultimap.create(); // createAbstractTaskAdapters(tasks);

		// Re-enable annotation view updates.
		setUpdateEnabled(annotationUpdateEnabled);
	}

	/**
	 * Returns the (first) {@link ITaskAdapter} which is associated with the given
	 * AF3 {@link Component}. Returns null if no matching {@link ITaskAdapter} is
	 * found.
	 */
	public ITaskAdapter getTaskAdapter(Task af3Task) {
		if(af3Task == null) {
			throw new IllegalArgumentException("Cannot identify ITaskAdapter for 'null' Tasks");
		}
		ITaskAdapter taskAdapter = null;
		for(ITaskAdapter task : tasks) {
			if(task.getObject(Task.class) == af3Task) {
				taskAdapter = task;
				break;
			}
		}
		return taskAdapter;
	}
}
