/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.platform.model.ExecutionUnit;

import com.google.inject.Inject;

/**
 * A mapper that associates input {@link ExecutionUnit}s with their {@link IExecutionUnitAdapter}
 * abstraction used in the DSE.
 * 
 * @author diewald
 */
public class ExecUnitMapper {

	/** Internal association map. */
	private Map<ExecutionUnit, IExecutionUnitAdapter> execUnitMap = new HashMap<>();

	/** Constructor. */
	@Inject
	public ExecUnitMapper(Set<IExecutionUnitAdapter> execUnits) {
		for(IExecutionUnitAdapter execUnitAdp : execUnits) {
			execUnitMap.put(execUnitAdp.getObject(ExecutionUnit.class), execUnitAdp);
		}
	}

	/** Returns the {@link ITaskAdapter} corresponding to the given {@code task}. */
	public IExecutionUnitAdapter getAdapter(ExecutionUnit execUnit) {
		return execUnitMap.get(execUnit);
	}
}
