/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Decodes;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.Partition;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.partition.variable.TaskPartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.common.MappingDecoderBase;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Decoder;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Provides a {@link TaskPartitionMappingEncoding} to the set of {@link Phenotype} types where the
 * resources are updated to match the set of {@link Partition}s of an input
 * {@link PartitionMappingEncoding}.
 * This {@link Decoder} is an identity operation.
 * 
 * @author diewald
 */
@Singleton
public class TaskPartitionMappingIdentityDecoder extends
		MappingDecoderBase<TaskPartitionMappingEncoding, ITaskAdapter, Partition, TaskMappingEntry<ITaskAdapter, Partition>, TaskPartitionMappingEncoding> {

	/**
	 * Decodes/Updates the given {@link TaskPartitionMappingEncoding} with the number of
	 * additional resources of an update {@link PartitionMappingEncoding}.
	 */
	@Provides
	@Decodes
	public TaskPartitionMappingEncoding decode(@Genotyped TaskPartitionMappingEncoding tpmEnc,
			@Genotyped PartitionMappingEncoding partEnc) {
		return alignWithResourceModifications(tpmEnc, partEnc.getRequesters());
	}

	/** Returns the {@link TaskPartitionMappingEncoding} as its interface. */
	@SuppressWarnings("unchecked")
	@Provides
	@Decodes
	public <S extends ITaskAdapter, T extends IExecutionUnitAdapter, E extends TaskMappingEntry<S, T>>
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>
			decode(TaskPartitionMappingEncoding tpmEnc) {
		return (IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>)(IMappingEncoding<?, ?, ?, ?>)tpmEnc;
	}

	/** {@inheritDoc} */
	@Override
	public TaskPartitionMappingEncoding decode(Genotype genotype) {
		return null;
	}
}
