/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.transformation;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.Partition;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEntry;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.partition.model.allocation.PartitionToExecutionUnitAllocationEntry;
import org.fortiss.af3.platform.model.ExecutionUnit;

import com.google.inject.Provides;

/**
 * Transforms a {@link PartitionMappingEncoding} to a {@link SuperSet} of AF3
 * {@link PartitionToExecutionUnitAllocationEntry}s.
 * 
 * @author diewald
 */
public class PartitionToExecutionUnitTransformation extends
		ExplorationTransformationModuleBase<SuperSet<PartitionToExecutionUnitAllocationEntry>> {
	/**
	 * See {@link PartitionToExecutionUnitTransformation}.
	 * 
	 * @param pmEncoding
	 *            allocations of {@link Partition}s to {@link IExecutionUnitAdapter}s.
	 * @param af3Partitions
	 *            Required to declare the dependency on the {@link PartitionTransformation}.
	 * @param internalExtAssocMap
	 *            associations between the internal and AF3 partitions.
	 * @return SuperSet of AF3 Partitions.
	 */
	@Transforms
	@Provides
	public SuperSet<PartitionToExecutionUnitAllocationEntry> transform(
			@Genotyped PartitionMappingEncoding pmEncoding,
			SuperSet<org.fortiss.af3.partition.model.Partition> af3Partitions,
			Map<Partition, org.fortiss.af3.partition.model.Partition> internalExtAssocMap) {
		SuperSet<PartitionToExecutionUnitAllocationEntry> allocations =
				createSuperSet("PartToExU", PartitionToExecutionUnitAllocationEntry.class);
		for(PartitionMappingEntry entry : pmEncoding.getMappingEntries()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					internalExtAssocMap.get(entry.getSource());
			assert (af3Partition != null) : "Error in the mapping of internal Partitions --> AF3 Partitions.";
			PartitionToExecutionUnitAllocationEntry af3AllocEntry =
					createAllocationEntry(PartitionToExecutionUnitAllocationEntry.class);
			af3AllocEntry.setSourceElement(af3Partition);
			af3AllocEntry.getTargetElements().add(entry.getTarget().getObject(ExecutionUnit.class));
			allocations.getEntries().add(af3AllocEntry);
		}
		return allocations;
	}
}
