/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable;

import static java.util.Collections.emptySet;
import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.module.common.encoding.MappingEncodingBase;
import org.fortiss.af3.exploration.alg.module.common.operator.copy.Copyable;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

/**
 * Encoding that maps partition, which host tasks, to execution units. Typically, these are cores.
 * The encoding is essentially a two-dimensional table.
 * 
 * @author diewald
 */
public class PartitionMappingEncoding extends
		MappingEncodingBase<Partition, IExecutionUnitAdapter, PartitionMappingEntry, PartitionMappingEncoding>
		implements Copyable<PartitionMappingEncoding> {

	/** Constructor. */
	public PartitionMappingEncoding(Set<Partition> sourceSet,
			Set<IExecutionUnitAdapter> targetSet,
			Collection<InternalConstraint<?>> constraints) {
		super(sourceSet, targetSet, constraints);
	}

	/** Prototype Constructor. */
	private PartitionMappingEncoding() {
		// Prototype for newInstance().
		super(emptySet(), null, null);
	}

	/** Copy constructor. */
	protected PartitionMappingEncoding(PartitionMappingEncoding encoding) {
		super(encoding);
	}

	/** {@inheritDoc} */
	@Override
	public PartitionMappingEntry createEntry(Partition requester,
			IExecutionUnitAdapter resource) {
		return new PartitionMappingEntry(requester, resource);
	}

	/** Removes a random {@link Partition} allocation to the given {@link IResourceAdapter}. */
	public boolean removeRandomPartition(IExecutionUnitAdapter resource) {
		List<Partition> partitions = getRequesters(resource);
		if(partitions.size() == 1) {
			return false;
		}
		int randIdx = rand.nextInt(partitions.size());
		Partition rmPartition = partitions.get(randIdx);
		// There exists at least one mapping entry, since we are operating on the list of partitions
		// assigned to the given resource.
		PartitionMappingEntry rmEntry = getFirst(getMappingEntriesOf(rmPartition)).get();
		removeEntry(rmEntry);
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public int size() {
		return allocationTable.size();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public PartitionMappingEncoding newInstance() {
		return new PartitionMappingEncoding();
	}

	/** {@inheritDoc} */
	@Override
	public PartitionMappingEncoding copy() {
		return new PartitionMappingEncoding(this);
	}
}
