/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.TaskGraphEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.Genotype;

/**
 * Concrete {@link TaskMappingEncodingBase} that maps {@link ITaskAdapter}s to {@link Partition}s.
 * 
 * @author diewald
 */
public class TaskPartitionMappingEncoding extends
		TaskMappingEncodingBase<ITaskAdapter, Partition, TaskMappingEntry<ITaskAdapter, Partition>, TaskPartitionMappingEncoding> {

	/** Constructor that optionally initializes the encoding to a random encoding. */
	public TaskPartitionMappingEncoding(Set<Partition> resources,
			TaskGraphEncodingBase<?, ITaskAdapter> tgEncoding,
			Collection<InternalConstraint<?>> constraints, boolean initialize) {
		super(resources, tgEncoding, constraints, initialize);
	}

	/** Copy Constructor. */
	protected TaskPartitionMappingEncoding(TaskPartitionMappingEncoding encoding) {
		super(encoding);
	}

	/** {@inheritDoc} */
	@Override
	public TaskMappingEntry<ITaskAdapter, Partition> createEntry(ITaskAdapter requester,
			Partition resource) {
		return new TaskMappingEntry<ITaskAdapter, Partition>(requester, resource);
	}

	/** {@inheritDoc} */
	@Override
	public <G extends Genotype> G newInstance() {
		// Not needed: not a list Genotype.
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public TaskPartitionMappingEncoding copy() {
		return new TaskPartitionMappingEncoding(this);
	}
}
