/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable.operators;

import org.fortiss.af3.exploration.alg.dse.RandomExploration;
import org.fortiss.af3.exploration.alg.feature.partition.variable.TaskPartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.module.common.operator.mutate.mapping.MutateAllocation;
import org.fortiss.af3.exploration.alg.module.common.operator.mutate.mapping.MutateRedundancyDecrement;
import org.fortiss.af3.exploration.alg.module.common.operator.mutate.mapping.MutateRedundancyIncrement;
import org.opt4j.operators.mutate.BasicMutateModule;
import org.opt4j.operators.mutate.Mutate;
import org.opt4j.operators.selection.RandomOperatorSelector;

import com.google.inject.Module;

/**
 * This {@link Module} registers {@link Mutate} operators in Opt4J which manipulate the
 * {@link TaskMappingEncodingBase}s.
 * 
 * @author huang
 * @author diewald
 */
public class MutateTaskPartitionMappingModule extends BasicMutateModule {

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void config() {
		super.config();

		addOperatorSelectorInstance(TaskPartitionMappingEncoding.class, Mutate.class,
				new RandomOperatorSelector(RandomExploration.getInstance()));
		addOperator(TaskPartitionMappingEncoding.class,
				(Class<? extends Mutate<TaskPartitionMappingEncoding>>)MutateAllocation.class);
		addOperator(TaskPartitionMappingEncoding.class,
				(Class<? extends Mutate<TaskPartitionMappingEncoding>>)MutateRedundancyDecrement.class);
		addOperator(TaskPartitionMappingEncoding.class,
				(Class<? extends Mutate<TaskPartitionMappingEncoding>>)MutateRedundancyIncrement.class);
	}
}
