/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.transform;

import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.IPlatformArchitectureElement;
import org.fortiss.af3.platform.model.PlatformConnectorUnit;
import org.fortiss.af3.platform.model.TransmissionConnection;

import com.google.inject.Provides;

/**
 * Adds all {@link PlatformConnectorUnit} of {@link IPlatformArchitectureElement}s that are already
 * present in the target model to a {@link SuperSet} of {@link PlatformConnectorUnit}s. Thereby,
 * such connectors without a containment are not deleted when returning the exploration results.
 * 
 * @author diewald
 */
public class PlatformConnectorTransformer
		extends ExplorationTransformationModuleBase<SuperSet<PlatformConnectorUnit>> {

	/**
	 * See {@link PlatformConnectorTransformer}.
	 * 
	 * @param connectionSuperSet
	 *            previously constructed {@link SuperSet} of {@link TransmissionConnection}s to
	 *            extract the created ports.
	 * @return SuperSet of AF3 {@link ExecutionUnit} instances.
	 */
	@Transforms
	@Provides
	public <N extends EObject, S extends EObject> SuperSet<PlatformConnectorUnit>
			transform(SuperSet<TransmissionConnection> connectionSuperSet) {
		SuperSet<PlatformConnectorUnit> connectors =
				createSuperSet("PlatformConnectorUnits", PlatformConnectorUnit.class);
		for(TransmissionConnection tConn : connectionSuperSet.getEntries()) {
			PlatformConnectorUnit srcConnector = tConn.getSource();
			if(srcConnector.eContainer() == null) {
				connectors.getEntries().add(srcConnector);
			}
			PlatformConnectorUnit tgtConnector = tConn.getTarget();
			if(tgtConnector.eContainer() == null) {
				connectors.getEntries().add(tConn.getTarget());
			}
		}
		return connectors;
	}
}
