/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.transform;

import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;
import static org.fortiss.af3.exploration.util.ExplorationModelElementFactory.createInstanceOfElementSpec;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.copy;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.variable.PlatformExecUnitInstanceEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.TaskToExUTransformation;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.exploration.model.specification.InstanceOfElement;
import org.fortiss.af3.platform.model.ExecutionUnit;

import com.google.inject.Provides;

/**
 * Transforms a {@link PlatformExecUnitInstanceEncoding} into a {@link SuperSet} of
 * {@link ExecutionUnit}s that contains all generated instances.
 * 
 * @author diewald
 */
public class PlatformExecutionUnitTransformer
		extends ExplorationTransformationModuleBase<SuperSet<ExecutionUnit>> {

	/**
	 * See {@link TaskToExUTransformation}.
	 * 
	 * @param peuiEnc
	 *            encoding relating a parent {@link ExecutionUnit} to is contained instances of
	 *            {@link ExecutionUnit}s.
	 * @return SuperSet of AF3 {@link ExecutionUnit} instances.
	 */
	@Transforms
	@Provides
	public <N extends EObject, S extends EObject> SuperSet<ExecutionUnit>
			transform(@Genotyped PlatformExecUnitInstanceEncoding peuiEnc) {
		SuperSet<ExecutionUnit> execUnits = createSuperSet("ExecUnits", ExecutionUnit.class);
		for(IExecutionUnitAdapter parentExecUnitAdp : peuiEnc.getAbstractContainerExecUnits()) {
			IExecutionUnitAdapter templateAdp = peuiEnc.getTemplateOf(parentExecUnitAdp);
			ExecutionUnit template = templateAdp.getObject(ExecutionUnit.class);

			for(IExecutionUnitAdapter execUnitAdp : peuiEnc.getChildInstances(parentExecUnitAdp)) {
				ExecutionUnit execUnit = (ExecutionUnit)copy(execUnitAdp.getObject(EObject.class));
				execUnitAdp.setObject(execUnit, ExecutionUnit.class);

				InstanceOfElement instanceSpec = createInstanceOfElementSpec(execUnit, template);
				execUnit.addSpecification(instanceSpec);

				execUnits.getEntries().add(execUnit);
			}
		}

		return execUnits;
	}
}
