/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.safetyarch.variable;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.Creates;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISafetyFunctionAdapter;
import org.fortiss.af3.exploration.alg.module.CreatorModule;

import com.google.inject.Provides;

/**
 * Creator for the {@link SafetyFunctionArchEncoding} that provides a standard encoding for the
 * exploration of safety function designs.
 * 
 * @author diewald
 */
public class SafetyFunctionArchCreator extends CreatorModule<SafetyFunctionArchEncoding> {

	/**
	 * Creates a {@link SafetyFunctionArchEncoding} encoding the architecture of safety functions,
	 * e.g., 1oo2, for all safety functions present in the input system.
	 */
	@Provides
	@Creates
	@Genotyped
	public SafetyFunctionArchEncoding create(Collection<ISafetyFunctionAdapter> sfAdapters) {
		// It is clear that the generic is right. However, the compiler does not get it. Thus,
		// suppress the corresponding Warnings.
		return new SafetyFunctionArchEncoding(sfAdapters, true, true);
	}

	/** {@inheritDoc} */
	@Override
	public SafetyFunctionArchEncoding create() {
		// Required by the framework...
		return null;
	}
}
