/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling.scheduler;

import java.util.Map;

import org.apfloat.Apcomplex;
import org.apfloat.Apfloat;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.scheduling.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.feature.scheduling.TimeSlot;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.AcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;

/**
 * Simple Multicore EDF task scheduler.
 * 
 * @author huang
 * @author barner
 * @author diewald
 */
public class EDFScheduler extends
		EDFSchedulerBase<IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>> {

	/**
	 * Returns the execution time for the current
	 * {@link org.fortiss.af3.exploration.alg.feature.scheduling.scheduler.EDFSchedulerBase.SchedulingObject}
	 * .
	 */
	@Override
	protected Apfloat getExecutionTime(
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> encoding,
			AcyclicTaskGraphEncoding acycItgEnc, ITaskAdapter task,
			TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter> entry) {
		Apfloat executiontime = new Apfloat(task.getWcet(entry.getTarget()), DOUBLE_PRECISION);
		return executiontime;
	}

	/** {@inheritDoc} */
	@Override
	protected Apfloat registerIncomingMessagesWithSchedule(
			IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?> encoding,
			AcyclicTaskGraphEncoding acycItgEnc,
			StrictTTSchedule<IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, IMappingEncoding<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>> schedule,
			Map<TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, TimeSlot> mappingEntryToTimeSlot,
			ITaskAdapter deployableComponent, Apfloat starttime,
			IExecutionUnitAdapter deploymentTarget, long iteration, TimeSlot slot) {
		return Apcomplex.ZERO;
	}
}
