/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskinstantiation;

import static java.util.Arrays.asList;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.feature.ExplorationFeature;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.problem.InstantiatedAcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.problem.InstantiatedTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.problem.TaskMappingToAcyclicGraphDecoder;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.problem.TaskMappingToGraphDecoder;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable.TaskInstanceCreator;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable.TaskInstanceEncoding;
import org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable.TaskInstanceMutateModule;

import com.google.inject.Injector;
import com.google.inject.TypeLiteral;

/**
 * Exploration feature for systems where {@link Component}s can be "abstract", i.e. they are like
 * interfaces, and have to be substituted by one out of N designs that represent an implementation
 * of the interface.
 * 
 * @author diewald
 */
public class DesignDiversityExploration extends ExplorationFeature {

	/** Constructor. */
	public DesignDiversityExploration() {
		super("Exploration of diversely designed tasks");

		addVariable(TypeLiteral.get(TaskInstanceEncoding.class), new TaskInstanceCreator(),
				asList(new TaskInstanceMutateModule()));

		addProblem(TypeLiteral.get(InstantiatedTaskGraphEncoding.class),
				new TaskMappingToGraphDecoder());
		addProblem(TypeLiteral.get(InstantiatedAcyclicTaskGraphEncoding.class),
				new TaskMappingToAcyclicGraphDecoder());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		return true;
	}
}
