/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable;

import java.util.List;
import java.util.Random;

import org.fortiss.af3.exploration.alg.dse.RandomExploration;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.Creates;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.module.CreatorModule;

import com.google.common.collect.Multimap;
import com.google.inject.Provides;

/**
 * Creates a {@link TaskInstanceEncoding} from scratch.
 * 
 * @author diewald
 */
public class TaskInstanceCreator extends CreatorModule<TaskInstanceEncoding> {

	/** References the DSE-internal {@link Random} number generator. */
	RandomExploration rand = RandomExploration.getInstance();

	/**
	 * Creates a {@link TaskInstanceEncoding} from a {@link Multimap} that associates abstract tasks
	 * with their possible instances.
	 */
	@Provides
	@Creates
	@Genotyped
	public TaskInstanceEncoding create(Multimap<ITaskAdapter, ITaskAdapter> instanceMap) {
		TaskInstanceEncoding encoding = new TaskInstanceEncoding(instanceMap);
		for(ITaskAdapter abstractTask : encoding.getAbstractElements()) {
			List<ITaskAdapter> availInstances = encoding.getAvailableInstaces(abstractTask);
			int selIdx = rand.nextInt(availInstances.size());
			encoding.setInstanceOf(abstractTask, availInstances.get(selIdx));
		}
		return encoding;
	}

	/** {@inheritDoc} */
	@Override
	public TaskInstanceEncoding create() {
		return null;
	}
}
