/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskExUMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskExUMappingEntry;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.partition.model.allocation.TaskToPartitionAllocationEntry;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.task.model.Task;
import org.fortiss.af3.task.model.allocation.TaskToExecutionUnitAllocationEntry;

import com.google.inject.Provides;

/**
 * Transforms a {@link TaskMappingEncodingBase} to a {@link SuperSet} of AF3
 * {@link TaskToPartitionAllocationEntry}s.
 * 
 * @author diewald
 */
public class TaskToExUTransformation
		extends ExplorationTransformationModuleBase<SuperSet<TaskToExecutionUnitAllocationEntry>> {

	/**
	 * See {@link TaskToExUTransformation}.
	 * 
	 * @param taskMapping
	 *            allocations of {@link ITaskAdapter}s to {@link IExecutionUnitAdapter}s.
	 * @return SuperSet of AF3 Partitions.
	 */
	@Transforms
	@Provides
	public <N extends EObject, S extends EObject> SuperSet<TaskToExecutionUnitAllocationEntry>
			transform(TaskExUMappingEncoding taskMapping) {
		SuperSet<TaskToExecutionUnitAllocationEntry> allocations =
				createSuperSet("TaskToExecUnit", TaskToExecutionUnitAllocationEntry.class);
		for(TaskExUMappingEntry entry : taskMapping.getMappingEntries()) {
			TaskToExecutionUnitAllocationEntry af3AllocEntry =
					createAllocationEntry(TaskToExecutionUnitAllocationEntry.class);
			af3AllocEntry.getSourceElements().add(entry.getSource().getObject(Task.class));
			af3AllocEntry.setTargetElement(entry.getTarget().getObject(ExecutionUnit.class));
			allocations.getEntries().add(af3AllocEntry);
		}
		return allocations;
	}
}
