/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping.evaluator.internal;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.evaluate.annotations.Evaluate;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.evaluate.annotations.Validates;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskExUMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.module.common.evaluator.BooleanValue;
import org.fortiss.af3.exploration.alg.module.common.evaluator.ConstraintEvaluatorBase;
import org.fortiss.af3.exploration.moea.model.constraints.AssignmentConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.problem.Evaluator;

import com.google.inject.Provides;

/**
 * {@link Evaluator} for {@link AssignmentConstraint}s that are {@link InternalConstraint}s. Hence,
 * this {@link Evaluator} only provides a meaningful validate method.
 * 
 * @author diewald
 */
public class TaskExUAssignmentConstraintEvaluator extends
		ConstraintEvaluatorBase<Boolean, AssignmentConstraint, TaskMappingEncodingBase<?, ?, ?, ?>> {

	/** Prototype Constructor. */
	public TaskExUAssignmentConstraintEvaluator(Class<AssignmentConstraint> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Constructor. */
	public TaskExUAssignmentConstraintEvaluator(AssignmentConstraint explorationTarget) {
		super(explorationTarget);
	}

	/** Dummy evaluation. */
	@Provides
	@Evaluate(TaskExUAssignmentConstraintEvaluator.class)
	public BooleanValue evaluateGoal() {
		return new BooleanValue(false);
	}

	/**
	 * Validates that {@link IRequestAdapter}s are only mapped to {@link IResourceAdapter}s that are
	 * allowed by the given constraint.
	 */
	@Provides
	@Validates
	public Boolean validate(TaskExUMappingEncoding tmEnc) {
		// TODO(AD): The assignment constraints must be translated in the preprocessing to produce
		// multiple assignment constraints that match the enabled features to consider multi-layer
		// mappings.
		for(Object reqObj : explorationTarget.getSources()) {
			IRequestAdapter reqAdp = (IRequestAdapter)reqObj;
			@SuppressWarnings("unchecked") Collection<IResourceAdapter> targetAdp =
					(Collection<IResourceAdapter>)(Collection<?>)tmEnc
							.getResources((ITaskAdapter)reqAdp);
			if(!(explorationTarget.getTargets().containsAll(targetAdp))) {
				return false;
			}
		}
		return true;
	}
}
