/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common;

import static org.eclipse.core.runtime.Assert.isTrue;
import static org.fortiss.af3.exploration.alg.util.ExplorationAlgUtils.DSE_VALIDATE_ENCODING;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.RandomExploration;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.module.DecoderModule;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEncoding;
import org.fortiss.af3.exploration.alg.module.common.encoding.IMappingEntry;

/**
 * Base class that aligns a task mapping with resource modifications.
 * 
 * @author diewald
 */
public abstract class MappingDecoderBase<P extends Phenotype, S extends IRequestAdapter, T extends IResourceAdapter, E extends IMappingEntry<S, T>, M extends IMappingEncoding<S, T, E, M>>
		extends DecoderModule<P> {

	/** References the DSE-internal {@link Random} number generator. */
	private RandomExploration rand = RandomExploration.getInstance();

	/**
	 * Reallocates the {@link IResourceAdapter}s if the set of {@link IResourceAdapter}s was changed
	 * by a previous module. This is required to stay consistent.
	 * 
	 * @param mapping
	 *            {@link TaskMappingEncodingBase} to adapt.
	 * @param updatedResources
	 *            Set of updated resource (mapping targets).
	 * @return Aligned {@link TaskMappingEncodingBase}.
	 */
	protected M alignWithResourceModifications(M mapping, Set<T> updatedResources) {
		if(DSE_VALIDATE_ENCODING) {
			isTrue(mapping.isValidAllocations());
		}
		mapping.updateValidAllocations(mapping.getRequesters(), updatedResources,
				constraints.get());
		Collection<E> relocateEntries = new ArrayList<>();
		for(E mappingEntry : mapping.getMappingEntries()) {
			if(!updatedResources.contains((mappingEntry).getTarget())) {
				relocateEntries.add(mappingEntry);
			}
		}

		if(DSE_VALIDATE_ENCODING) {
			isTrue(mapping.isValidAllocations());
		}

		for(E mappingEntry : relocateEntries) {
			List<T> validTargets = mapping.getValidTargetResources(mappingEntry.getSource());
			int selIdx = rand.nextInt(validTargets.size());
			mapping.resetResourceOf(mappingEntry, validTargets.get(selIdx));
		}

		if(DSE_VALIDATE_ENCODING) {
			isTrue(mapping.isValidAllocations());
		}
		return mapping;
	}
}
