/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.encoding;

import java.util.Collection;
import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IModelElementAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.service.module.IExplorationEncoding;
import org.opt4j.core.optimizer.Operator;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;

/**
 * Common interface for {@link IExplorationEncoding}s
 * 
 * @author diewald
 */
public abstract class InstanceEncodingBase<T extends IModelElementAdapter>
		implements IInstanceEncoding<T> {

	/** References all possible implementations of a given {@link ITaskAdapter}. */
	protected ListMultimap<T, T> availableInstances = ArrayListMultimap.create();

	/** Marks an encoding infeasible if an {@link Operator} detects a constraint violation. */
	private boolean isInfeasible = false;

	/** {@inheritDoc} */
	@Override
	public Collection<T> getAbstractElements() {
		return availableInstances.values();
	}

	/** {@inheritDoc} */
	@Override
	public List<T> getAvailableInstaces(T abstractElement) {
		return availableInstances.get(abstractElement);
	}

	/** Returns isInvalid. */
	@Override
	public boolean isInfeasible() {
		return isInfeasible;
	}

	/** {@inheritDoc} */
	@Override
	public void setInfeasible() {
		isInfeasible = true;
	}
}
