/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import static java.util.stream.Collectors.toSet;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.AcyclicTaskGraph;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.AcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.TaskGraphEncoding;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Constructs a {@link AcyclicTaskGraphEncoding} from an input task architecture that contains the
 * set of independent {@link TaskGraph}s. In difference to the {@link TaskGraphEncoding}, this
 * encoding contains only task graphs that do not contain cycles. This is required for scheduling.
 * 
 * @author diewald
 */
public class TaskGraphAcyclicInputModule extends InputModule<AcyclicTaskGraphEncoding> {

	/** See {@link TaskGraphInputModule}. */
	@InputProvider
	@Provides
	@Singleton
	public AcyclicTaskGraphEncoding getAcyclicTaskGraphEnc(TaskGraphEncoding taskGraphEnc) {
		Collection<TaskGraph> taskGraphs = taskGraphEnc.getTaskGraphs();
		Set<AcyclicTaskGraph> acycTgs =
				taskGraphs.stream().map(tg -> new AcyclicTaskGraph(tg)).collect(toSet());
		return new AcyclicTaskGraphEncoding(acycTgs, taskGraphEnc.getHyperPeriod());
	}
}
