/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module.base;

import java.util.Collection;
import java.util.function.Predicate;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.service.module.IExpSpecTransformationModule;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;

import com.google.common.reflect.TypeToken;

/**
 * Base class for {@link IExpSpecTransformationModule}s that declares the in- and output types.
 * 
 * @author diewald
 */
public abstract class ExpSpecTransformatorBase<I extends EObject, O extends EObject>
		implements IExpSpecTransformationModule<I, O> {

	/** Boilerplate: "Remembers" the type of the input given as a generic. */
	private final TypeToken<I> inTypeToken = new TypeToken<I>(getClass()) { // ...
	};

	/** Boilerplate: "Remembers" the type of the output given as a generic. */
	private final TypeToken<O> outTypeToken = new TypeToken<O>(getClass()) { // ...
	};

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<I> getInputType() {
		return (Class<I>)inTypeToken.getRawType();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<O> getOutputType() {
		return (Class<O>)outTypeToken.getRawType();
	}

	/** {@inheritDoc} */
	@Override
	public Predicate<I> getInputFilter() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IIntrospectionItem> getIntrospectionItems() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		return null;
	}
}
