/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module.base.transformation;

import java.lang.reflect.ParameterizedType;

import org.fortiss.af3.exploration.alg.service.module.IExplorationTransformationModule;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.common.reflect.TypeToken;
import com.google.inject.AbstractModule;
import com.google.inject.TypeLiteral;

/**
 * Base class for {@link IExplorationTransformationModule}s.
 * 
 * @author diewald
 */
public abstract class ExplorationTransformationModuleBase<T extends SuperSet<? extends IModelElement>>
		extends AbstractModule implements IExplorationTransformationModule<T> {

	/** Output {@link SuperSet} type of this transformer. */
	protected TypeLiteral<T> outputType;

	/** Constructor. */
	@SuppressWarnings("unchecked")
	public ExplorationTransformationModuleBase() {
		TypeToken<T> typeToken = new TypeToken<T>(getClass()) {
			/**/};
		ParameterizedType paramType = (ParameterizedType)typeToken.getType();
		outputType = (TypeLiteral<T>)TypeLiteral.get(paramType);
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<T> getProvidedType() {
		return outputType;
	}
}
