/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.util;

import org.fortiss.af3.exploration.dseml.model.booleanp.Exists;
import org.fortiss.af3.exploration.dseml.model.booleanp.ForAll;
import org.fortiss.af3.exploration.dseml.model.booleanp.allocation.ILocationExpression;
import org.fortiss.af3.exploration.dseml.model.expression.IExpression;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationTarget;

/**
 * Utility methods for handling DSML {@link IExpression}s of {@link ExplorationTarget}s
 * 
 * @author diewald
 */
public class DseMLUtils {

	/**
	 * Extracts the {@link ILocationExpression} of an allocation pattern
	 * {@link ExplorationConstraint} or {@code null} if the {@link ExplorationConstraint} is not
	 * such a pattern.
	 */
	public static ILocationExpression
			getDirectAllocationExpression(ExplorationConstraint<?> constraint) {
		IExpression expr = constraint.getExpression();
		if(!(expr instanceof ForAll)) {
			return null;
		}
		IExpression forAllExpr = ((ForAll)expr).getExpression();
		if(!(forAllExpr instanceof Exists)) {
			return null;
		}
		IExpression existsExpr = ((Exists)forAllExpr).getExpression();
		if(existsExpr instanceof ILocationExpression) {
			return (ILocationExpression)existsExpr;
		}
		return null;
	}

	/** Returns whether the given {@link ExplorationConstraint} is an allocation pattern. */
	public static boolean isAllocationPattern(ExplorationConstraint<?> constraint) {
		return getDirectAllocationExpression(constraint) != null;
	}
}
